@@ -16,21 +16,23 @@ local scope    = require 'workspace.scope'
 local lazy     = require 'lazytable'
 local cacher   = require 'lazy-cacher'
 local sp       = require 'bee.subprocess'
+local pub      = require 'pub'
 
 ---@class file
----@field uri          uri
----@field content      string
----@field _ref?        integer
----@field trusted?     boolean
----@field rows?        integer[]
----@field originText?  string
----@field text         string
----@field version?     integer
----@field originLines? integer[]
----@field state?       parser.state
----@field _diffInfo?   table[]
----@field cache        table
----@field id           integer
+---@field uri           uri
+---@field ref?          integer
+---@field trusted?      boolean
+---@field rows?         integer[]
+---@field originText?   string
+---@field text?         string
+---@field version?      integer
+---@field originLines?  integer[]
+---@field diffInfo?     table[]
+---@field cache?        table
+---@field id            integer
+---@field state?        parser.state
+---@field compileCount? integer
+---@field words?        table
 
 ---@class files
 ---@field lazyCache?   lazy-cacher
@@ -48,8 +50,10 @@ function m.reset()
     m.visible        = {}
     m.globalVersion  = 0
     m.fileCount      = 0
-    m.astCount       = 0
-    m.astMap         = {}
+    ---@type table<uri, parser.state>
+    m.stateMap       = setmetatable({}, util.MODE_V)
+    ---@type table<parser.state, true>
+    m.stateTrace     = setmetatable({}, util.MODE_K)
 end
 
 m.reset()
@@ -57,10 +61,29 @@ m.reset()
 local fileID = util.counter()
 
 local uriMap = {}
+
+---@param path fs.path
+---@return fs.path
+local function getRealParent(path)
+    local parent = path:parent_path()
+    if parent:string():gsub('^%w+:', string.lower)
+    == path  :string():gsub('^%w+:', string.lower) then
+        return path
+    end
+    local res = fs.fullpath(path)
+    return getRealParent(parent) / res:filename()
+end
+
 -- 获取文件的真实uri，但不穿透软链接
 ---@param uri uri
 ---@return uri
 function m.getRealUri(uri)
+    if platform.OS ~= 'Windows' then
+        return furi.normalize(uri)
+    end
+    if not furi.isValid(uri) then
+        return uri
+    end
     local filename = furi.decode(uri)
     -- normalize uri
     uri = furi.encode(filename)
@@ -78,11 +101,16 @@ function m.getRealUri(uri)
     if uri == ruri then
         return ruri
     end
+    local real = getRealParent(path:parent_path()) / res:filename()
+    ruri = furi.encode(real:string())
+    if uri == ruri then
+        return ruri
+    end
     if not uriMap[uri] then
-        uriMap[uri] = ruri
+        uriMap[uri] = true
         log.warn(('Fix real file uri: %s -> %s'):format(uri, ruri))
     end
-    return uriMap[uri]
+    return ruri
 end
 
 --- 打开文件
@@ -104,7 +132,7 @@ function m.close(uri)
     end
     m.onWatch('close', uri)
     if file then
-        if (file._ref or 0) <= 0 and not m.isOpen(uri) then
+        if (file.ref or 0) <= 0 and not m.isOpen(uri) then
             m.remove(uri)
         end
     end
@@ -163,7 +191,7 @@ end
 ---@return string
 local function pluginOnSetText(file, text)
     local plugin   = require 'plugin'
-    file._diffInfo = nil
+    file.diffInfo = nil
     local suc, result = plugin.dispatch('OnSetText', file.uri, text)
     if not suc then
         if DEVELOP and result then
@@ -177,7 +205,7 @@ local function pluginOnSetText(file, text)
         local diffs
         suc, result, diffs = xpcall(smerger.mergeDiff, log.error, text, result)
         if suc then
-            file._diffInfo = diffs
+            file.diffInfo = diffs
             file.originLines = parser.lines(text)
             return result
         else
@@ -189,6 +217,12 @@ local function pluginOnSetText(file, text)
     return text
 end
 
+---@param file file
+function m.removeState(file)
+    file.state = nil
+    m.stateMap[file.uri] = nil
+end
+
 --- 设置文件文本
 ---@param uri uri
 ---@param text? string
@@ -228,15 +262,16 @@ function m.setText(uri, text, isTrust, callback)
     if file.originText == text then
         return
     end
+    local clock = os.clock()
     local newText = pluginOnSetText(file, text)
-    file.text       = newText
-    file.trusted    = isTrust
-    file.originText = text
-    file.rows       = nil
-    file.words      = nil
-    m.astMap[uri]   = nil
-    file.cache = {}
-    file.cacheActiveTime = math.huge
+    m.removeState(file)
+    file.text            = newText
+    file.trusted         = isTrust
+    file.originText      = text
+    file.rows            = nil
+    file.words           = nil
+    file.compileCount    = 0
+    file.cache           = {}
     m.globalVersion = m.globalVersion + 1
     m.onWatch('version', uri)
     if create then
@@ -250,6 +285,7 @@ function m.setText(uri, text, isTrust, callback)
             util.saveFile(LOGPATH .. '/diffed.lua', newText)
         end
     end
+    log.trace('Set text:', uri, 'takes', os.clock() - clock, 'sec.')
 
     --if instance or TEST then
     --else
@@ -266,6 +302,9 @@ end
 
 function m.resetText(uri)
     local file = m.fileMap[uri]
+    if not file then
+        return
+    end
     local originText = file.originText
     file.originText = nil
     m.setText(uri, originText, file.trusted)
@@ -278,7 +317,7 @@ function m.setRawText(uri, text)
     local file = m.fileMap[uri]
     file.text             = text
     file.originText       = text
-    m.astMap[uri]         = nil
+    m.removeState(file)
 end
 
 function m.getCachedRows(uri)
@@ -368,6 +407,16 @@ function m.getOriginText(uri)
     return file.originText
 end
 
+---@param uri uri
+---@param text string
+function m.setOriginText(uri, text)
+    local file = m.fileMap[uri]
+    if not file then
+        return
+    end
+    file.originText = text
+end
+
 --- 获取文件原始行表
 ---@param uri uri
 ---@return integer[]
@@ -395,8 +444,8 @@ function m.addRef(uri)
     if not file then
         return nil
     end
-    file._ref = (file._ref or 0) + 1
-    log.debug('add ref', uri)
+    file.ref = (file.ref or 0) + 1
+    log.debug('add ref', uri, file.ref)
     return function ()
         m.delRef(uri)
     end
@@ -407,9 +456,9 @@ function m.delRef(uri)
     if not file then
         return
     end
-    file._ref = (file._ref or 0) - 1
-    log.debug('del ref', uri)
-    if file._ref <= 0 and not m.isOpen(uri) then
+    file.ref = (file.ref or 0) - 1
+    log.debug('del ref', uri, file.ref)
+    if file.ref <= 0 and not m.isOpen(uri) then
         m.remove(uri)
     end
 end
@@ -421,8 +470,8 @@ function m.remove(uri)
     if not file then
         return
     end
+    m.removeState(file)
     m.fileMap[uri]        = nil
-    m.astMap[uri]         = nil
     m._pairsCache         = nil
 
     m.fileCount     = m.fileCount - 1
@@ -447,6 +496,7 @@ function m.getAllUris(suri)
             files[i] = uri
         end
     end
+    table.sort(files)
     return files
 end
 
@@ -489,12 +539,57 @@ function m.getLazyCache()
     return m.lazyCache
 end
 
-function m.compileState(uri, text)
+---@param state parser.state
+---@param file file
+function m.compileStateThen(state, file)
+    m.stateTrace[state] = true
+    m.stateMap[file.uri] = state
+    state.uri = file.uri
+    state.lua = file.text
+    state.ast.uri = file.uri
+    state.diffInfo   = file.diffInfo
+    state.originLines = file.originLines
+    state.originText  = file.originText
+
+    local clock = os.clock()
+    parser.luadoc(state)
+    local passed = os.clock() - clock
+    if passed > 0.1 then
+        log.warn(('Parse LuaDoc of [%s] takes [%.3f] sec, size [%.3f] kb.'):format(file.uri, passed, #file.text / 1000))
+    end
+
+    if LAZY and not file.trusted then
+        local cache = m.getLazyCache()
+        local id = ('%d'):format(file.id)
+        clock = os.clock()
+        state = lazy.build(state, cache:writterAndReader(id)):entry()
+        passed = os.clock() - clock
+        if passed > 0.1 then
+            log.warn(('Convert lazy-table for [%s] takes [%.3f] sec, size [%.3f] kb.'):format(file.uri, passed, #file.text / 1000))
+        end
+    end
+
+    file.compileCount = file.compileCount + 1
+    if file.compileCount >= 3 then
+        file.state = state
+        log.debug('State persistence:', file.uri)
+    end
+
+    m.onWatch('compile', file.uri)
+end
+
+---@param uri uri
+---@return boolean
+function m.checkPreload(uri)
+    local file = m.fileMap[uri]
+    if not file then
+        return false
+    end
     local ws     = require 'workspace'
     local client = require 'client'
     if  not m.isOpen(uri)
     and not m.isLibrary(uri)
-    and #text >= config.get(uri, 'Lua.workspace.preloadFileSize') * 1000 then
+    and #file.text >= config.get(uri, 'Lua.workspace.preloadFileSize') * 1000 then
         if not m.notifyCache['preloadFileSize'] then
             m.notifyCache['preloadFileSize'] = {}
             m.notifyCache['skipLargeFileCount'] = 0
@@ -505,7 +600,7 @@ function m.compileState(uri, text)
             local message = lang.script('WORKSPACE_SKIP_LARGE_FILE'
                         , ws.getRelativePath(uri)
                         , config.get(uri, 'Lua.workspace.preloadFileSize')
-                        , #text / 1000
+                        , #file.text / 1000
                     )
             if m.notifyCache['skipLargeFileCount'] <= 1 then
                 client.showMessage('Info', message)
@@ -513,92 +608,125 @@ function m.compileState(uri, text)
                 client.logMessage('Info', message)
             end
         end
-        return nil
+        return false
+    end
+    return true
+end
+
+---@param uri uri
+---@param callback fun(state: parser.state?)
+function m.compileStateAsync(uri, callback)
+    local file = m.fileMap[uri]
+    if not file then
+        callback(nil)
+        return
+    end
+    if m.stateMap[uri] then
+        callback(m.stateMap[uri])
+        return
+    end
+
+    ---@type brave.param.compile.options
+    local options = {
+        special           = config.get(uri, 'Lua.runtime.special'),
+        unicodeName       = config.get(uri, 'Lua.runtime.unicodeName'),
+        nonstandardSymbol = util.arrayToHash(config.get(uri, 'Lua.runtime.nonstandardSymbol')),
+    }
+
+    ---@type brave.param.compile
+    local params = {
+        uri     = uri,
+        text    = file.text,
+        mode    = 'Lua',
+        version = config.get(uri, 'Lua.runtime.version'),
+        options = options
+    }
+    pub.task('compile', params, function (result)
+        if file.text ~= params.text then
+            return
+        end
+        if not result.state then
+            log.error('Compile failed:', uri, result.err)
+            callback(nil)
+            return
+        end
+        m.compileStateThen(result.state, file)
+        callback(result.state)
+    end)
+end
+
+---@param uri uri
+---@return parser.state?
+function m.compileState(uri)
+    local file = m.fileMap[uri]
+    if not file then
+        return
+    end
+    if m.stateMap[uri] then
+        return m.stateMap[uri]
+    end
+    if not m.checkPreload(uri) then
+        return
+    end
+
+    ---@type brave.param.compile.options
+    local options = {
+        special           = config.get(uri, 'Lua.runtime.special'),
+        unicodeName       = config.get(uri, 'Lua.runtime.unicodeName'),
+        nonstandardSymbol = util.arrayToHash(config.get(uri, 'Lua.runtime.nonstandardSymbol')),
+    }
+
+    local ws     = require 'workspace'
+    local client = require 'client'
+    if not client.isReady() then
+        log.error('Client not ready!', uri)
     end
     local prog <close> = progress.create(uri, lang.script.WINDOW_COMPILING, 0.5)
     prog:setMessage(ws.getRelativePath(uri))
+    log.trace('Compile State:', uri)
     local clock = os.clock()
-    local state, err = parser.compile(text
+    local state, err = parser.compile(file.text
         , 'Lua'
         , config.get(uri, 'Lua.runtime.version')
-        , {
-            special           = config.get(uri, 'Lua.runtime.special'),
-            unicodeName       = config.get(uri, 'Lua.runtime.unicodeName'),
-            nonstandardSymbol = util.arrayToHash(config.get(uri, 'Lua.runtime.nonstandardSymbol')),
-        }
+        , options
     )
     local passed = os.clock() - clock
     if passed > 0.1 then
-        log.warn(('Compile [%s] takes [%.3f] sec, size [%.3f] kb.'):format(uri, passed, #text / 1000))
+        log.warn(('Compile [%s] takes [%.3f] sec, size [%.3f] kb.'):format(uri, passed, #file.text / 1000))
     end
-    --await.delay()
-    if state then
-        state.uri = uri
-        state.lua = text
-        state.ast.uri = uri
 
-        local clock = os.clock()
-        parser.luadoc(state)
-        local passed = os.clock() - clock
-        if passed > 0.1 then
-            log.warn(('Parse LuaDoc of [%s] takes [%.3f] sec, size [%.3f] kb.'):format(uri, passed, #text / 1000))
-        end
-
-        local file = m.fileMap[uri]
-        if LAZY and not file.trusted then
-            state.pushError = nil
-            local cache = m.getLazyCache()
-            local id = ('%d'):format(file.id)
-            clock = os.clock()
-            state = lazy.build(state, cache:writterAndReader(id)):entry()
-            passed = os.clock() - clock
-            if passed > 0.1 then
-                log.warn(('Convert lazy-table for [%s] takes [%.3f] sec, size [%.3f] kb.'):format(uri, passed, #text / 1000))
-            end
-        else
-            m.astCount = m.astCount + 1
-            local removed
-            setmetatable(state, {__gc = function ()
-                if removed then
-                    return
-                end
-                removed = true
-                m.astCount = m.astCount - 1
-            end})
-        end
-
-        return state
-    else
+    if not state then
         log.error('Compile failed:', uri, err)
         return nil
     end
+
+    m.compileStateThen(state, file)
+
+    return state
 end
 
+---@class parser.state
+---@field diffInfo? table[]
+---@field originLines? integer[]
+---@field originText? string
+---@field lua? string
+
 --- 获取文件语法树
 ---@param uri uri
----@return table? state
+---@return parser.state? state
 function m.getState(uri)
     local file = m.fileMap[uri]
     if not file then
         return nil
     end
-    local state = m.astMap[uri]
-    if not state then
-        state = m.compileState(uri, file.text)
-        m.astMap[uri] = state
-        file.state = state
-        --await.delay()
-    end
-    file.cacheActiveTime = timer.clock()
+    local state = m.compileState(uri)
     return state
 end
 
+---@param uri uri
+---@return parser.state?
 function m.getLastState(uri)
-    local file = m.fileMap[uri]
-    if not file then
-        return nil
-    end
-    return file.state
+    return m.stateMap[uri]
 end
 
 function m.getFile(uri)
@@ -618,43 +746,32 @@ local function isNameChar(text)
 end
 
 --- 将应用差异前的offset转换为应用差异后的offset
----@param uri    uri
+---@param state  parser.state
 ---@param offset integer
 ---@return integer start
 ---@return integer finish
-function m.diffedOffset(uri, offset)
-    local file = m.getFile(uri)
-    if not file then
+function m.diffedOffset(state, offset)
+    if not state.diffInfo then
         return offset, offset
     end
-    if not file._diffInfo then
-        return offset, offset
-    end
-    return smerger.getOffset(file._diffInfo, offset)
+    return smerger.getOffset(state.diffInfo, offset)
 end
 
 --- 将应用差异后的offset转换为应用差异前的offset
----@param uri    uri
+---@param state  parser.state
 ---@param offset integer
 ---@return integer start
 ---@return integer finish
-function m.diffedOffsetBack(uri, offset)
-    local file = m.getFile(uri)
-    if not file then
+function m.diffedOffsetBack(state, offset)
+    if not state.diffInfo then
         return offset, offset
     end
-    if not file._diffInfo then
-        return offset, offset
-    end
-    return smerger.getOffsetBack(file._diffInfo, offset)
+    return smerger.getOffsetBack(state.diffInfo, offset)
 end
 
-function m.hasDiffed(uri)
-    local file = m.getFile(uri)
-    if not file then
-        return false
-    end
-    return file._diffInfo ~= nil
+---@param state parser.state
+function m.hasDiffed(state)
+    return state.diffInfo ~= nil
 end
 
 --- 获取文件的自定义缓存信息（在文件内容更新后自动失效）
@@ -663,7 +780,6 @@ function m.getCache(uri)
     if not file then
         return nil
     end
-    --file.cacheActiveTime = timer.clock()
     return file.cache
 end
 
@@ -766,6 +882,50 @@ function m.getDllWords(uri)
     return file.words
 end
 
+---@return integer
+function m.countStates()
+    local n = 0
+    for _ in pairs(m.stateTrace) do
+        n = n + 1
+    end
+    return n
+end
+
+---@param path string
+---@return string
+function m.normalize(path)
+    path = path:gsub('%$%{(.-)%}', function (key)
+        if key == '3rd' then
+            return (ROOT / 'meta' / '3rd'):string()
+        end
+        if key:sub(1, 4) == 'env:' then
+            local env = os.getenv(key:sub(5))
+            return env
+        end
+    end)
+    path = util.expandPath(path)
+    path = path:gsub('^%.[/\\]+', '')
+    for _ = 1, 1000 do
+        if path:sub(1, 2) == '..' then
+            break
+        end
+        local count
+        path, count = path:gsub('[^/\\]+[/\\]+%.%.[/\\]', '/', 1)
+        if count == 0 then
+            break
+        end
+    end
+    if platform.OS == 'Windows' then
+        path = path:gsub('[/\\]+', '\\')
+                   :gsub('[/\\]+$', '')
+                   :gsub('^(%a:)$', '%1\\')
+    else
+        path = path:gsub('[/\\]+', '/')
+                   :gsub('[/\\]+$', '')
+    end
+    return path
+end
+
 --- 注册事件
 ---@param callback async fun(ev: string, uri: uri)
 function m.watch(callback)
@@ -780,23 +940,4 @@ function m.onWatch(ev, uri)
     end
 end
 
-function m.init()
-    --TODO 可以清空文件缓存，之后看要不要启用吧
-    --timer.loop(10, function ()
-    --    local list = {}
-    --    for _, file in pairs(m.fileMap) do
-    --        if timer.clock() - file.cacheActiveTime > 10.0 then
-    --            file.cacheActiveTime = math.huge
-    --            file.ast = nil
-    --            file.cache = {}
-    --            list[#list+1] = file.uri
-    --        end
-    --    end
-    --    if #list > 0 then
-    --        log.info('Flush file caches:', #list, '\n', table.concat(list, '\n'))
-    --        collectgarbage()
-    --    end
-    --end)
-end
-
 return m
