@@ -50,8 +50,10 @@ function m.create(uri)
     m.folders[#m.folders+1] = scp
     if uri == furi.encode '/'
     or uri == furi.encode(os.getenv 'HOME' or '') then
-        client.showMessage('Error', lang.script('WORKSPACE_NOT_ALLOWED', furi.decode(uri)))
-        scp:set('bad root', true)
+        if not FORCE_ACCEPT_WORKSPACE then
+            client.showMessage('Error', lang.script('WORKSPACE_NOT_ALLOWED', furi.decode(uri)))
+            scp:set('bad root', true)
+        end
     end
 end
 
@@ -84,26 +86,45 @@ function m.getRootUri(uri)
 end
 
 local globInteferFace = {
-    type = function (path)
+    type = function (path, data)
+        if data[path] then
+            return data[path]
+        end
         local result
         pcall(function ()
-            if fs.is_directory(path) then
+            if fs.is_directory(fs.path(path)) then
                 result = 'directory'
+                data[path] = 'directory'
             else
                 result = 'file'
+                data[path] = 'file'
             end
         end)
         return result
     end,
-    list = function (path)
-        local fullPath = fs.path(path)
-        if not fs.is_directory(fullPath) then
+    list = function (path, data)
+        if data[path] == 'file' then
             return nil
         end
+        local fullPath = fs.path(path)
+        if not fs.is_directory(fullPath) then
+            data[path] = 'file'
+            return nil
+        end
+        data[path] = true
         local paths = {}
         pcall(function ()
-            for fullpath in fs.pairs(fullPath) do
-                paths[#paths+1] = fullpath:string()
+            for fullpath, status in fs.pairs(fullPath) do
+                local pathString = fullpath:string()
+                paths[#paths+1] = pathString
+                local st = status:type()
+                if st == 'directory'
+                or st == 'symlink'
+                or st == 'junction' then
+                    data[pathString] = 'directory'
+                else
+                    data[pathString] = 'file'
+                end
             end
         end)
         return paths
@@ -198,14 +219,14 @@ function m.getLibraryMatchers(scp)
     for _, path in ipairs(config.get(scp.uri, 'Lua.workspace.library')) do
         path = m.getAbsolutePath(scp.uri, path)
         if path then
-            librarys[m.normalize(path)] = true
+            librarys[files.normalize(path)] = true
         end
     end
     local metaPaths = scp:get 'metaPaths'
     log.debug('meta path:', inspect(metaPaths))
     if metaPaths then
         for _, metaPath in ipairs(metaPaths) do
-            librarys[m.normalize(metaPath)] = true
+            librarys[files.normalize(metaPath)] = true
         end
     end
 
@@ -225,7 +246,7 @@ function m.getLibraryMatchers(scp)
     end
 
     scp:set('libraryMatcher', matchers)
-    log.debug('library matcher:', inspect(matchers))
+    --log.debug('library matcher:', inspect(matchers))
 
     return matchers
 end
@@ -307,9 +328,9 @@ function m.awaitPreload(scp)
 
     if scp.uri and not scp:get('bad root') then
         log.info('Scan files at:', scp:getName())
-        scp:gc(fw.watch(m.normalize(furi.decode(scp.uri)), true, function (path)
-            local uri = furi.encode(path)
-            if m.isIgnored(uri) and not files.isLibrary(uri) then
+        scp:gc(fw.watch(files.normalize(furi.decode(scp.uri)), true, function (path)
+            local rpath = m.getRelativePath(path)
+            if native(rpath) then
                 return false
             end
             return true
@@ -332,8 +353,8 @@ function m.awaitPreload(scp)
         log.info('Scan library at:', libMatcher.uri)
         local count = 0
         scp:gc(fw.watch(furi.decode(libMatcher.uri), true, function (path)
-            local uri = furi.encode(path)
-            if m.isIgnored(uri) and not files.isLibrary(uri) then
+            local rpath = m.getRelativePath(path)
+            if libMatcher.matcher(rpath) then
                 return false
             end
             return true
@@ -382,52 +403,18 @@ function m.findUrisByFilePath(path)
     return results
 end
 
----@param path string
----@return string
-function m.normalize(path)
-    path = path:gsub('%$%{(.-)%}', function (key)
-        if key == '3rd' then
-            return (ROOT / 'meta' / '3rd'):string()
-        end
-        if key:sub(1, 4) == 'env:' then
-            local env = os.getenv(key:sub(5))
-            return env
-        end
-    end)
-    path = util.expandPath(path)
-    path = path:gsub('^%.[/\\]+', '')
-    for _ = 1, 1000 do
-        if path:sub(1, 2) == '..' then
-            break
-        end
-        local count
-        path, count = path:gsub('[^/\\]+[/\\]+%.%.[/\\]', '/', 1)
-        if count == 0 then
-            break
-        end
-    end
-    if platform.OS == 'Windows' then
-        path = path:gsub('[/\\]+', '\\')
-                   :gsub('[/\\]+$', '')
-                   :gsub('^(%a:)$', '%1\\')
-    else
-        path = path:gsub('[/\\]+', '/')
-                   :gsub('[/\\]+$', '')
-    end
-    return path
-end
 
 ---@param folderUri? uri
 ---@param path string
 ---@return string?
 function m.getAbsolutePath(folderUri, path)
-    path = m.normalize(path)
+    path = files.normalize(path)
     if fs.path(path):is_relative() then
         if not folderUri then
             return nil
         end
         local folderPath = furi.decode(folderUri)
-        path = m.normalize(folderPath .. '/' .. path)
+        path = files.normalize(folderPath .. '/' .. path)
     end
     return path
 end
@@ -446,14 +433,14 @@ function m.getRelativePath(uriOrPath)
     end
     local scp = scope.getScope(uri)
     if not scp.uri then
-        local relative = m.normalize(path)
+        local relative = files.normalize(path)
         return relative:gsub('^[/\\]+', ''), false
     end
-    local _, pos = m.normalize(path):find(furi.decode(scp.uri), 1, true)
+    local _, pos = files.normalize(path):find(furi.decode(scp.uri), 1, true)
     if pos then
-        return m.normalize(path:sub(pos + 1)):gsub('^[/\\]+', ''), true
+        return files.normalize(path:sub(pos + 1)):gsub('^[/\\]+', ''), true
     else
-        return m.normalize(path):gsub('^[/\\]+', ''), false
+        return files.normalize(path):gsub('^[/\\]+', ''), false
     end
 end
 
@@ -484,10 +471,6 @@ function m.flushFiles(scp)
     for uri in pairs(cachedUris) do
         files.delRef(uri)
     end
-    collectgarbage()
-    collectgarbage()
-    -- TODO: wait maillist
-    collectgarbage 'restart'
 end
 
 ---@param scp scope
@@ -508,6 +491,8 @@ end
 ---@async
 ---@param scp scope
 function m.awaitReload(scp)
+    await.unique('workspace reload:' .. scp:getName())
+    await.sleep(0.1)
     scp:set('ready', false)
     scp:set('nativeMatcher', nil)
     scp:set('libraryMatcher', nil)
